% This is the method of lines  for the 1D Burgers equation
clear all; 
close all;
global c;   % scale of kernel
global VX   % the V_X matrix of the paper
global VXd  % the first derivative matrix
global VXdd % the second derivative matrix
global VL   % the V_\Lambda matrix of the paper
global VLd  % the first derivative 
global VLdd % the second derivative 
global boldg % boldface g of the paper
global Re    % Reynolds number
n=50;    % number of space discretization points
c=10/n; % first guess for scale of Gaussian
% Should be much too large to start with.
time=0;  % start time
Re=30;  
pts=(-1:2/(n-1):1)'; % spatial interpolation points.
amat=zeros(n,n); % the basic standard matrix
% The c is critical. We loop for it.
disp('Looping over c for a good scale')
while 1==1
    for i=1:n
        for j=1:n
            z=kernel1D(pts(i,1),pts(j,1));
            amat(i,j)=z(1); 
        end 
    end
    if condest(amat)<1.e12
        break;
    else
        c=c/1.2
    end
end
disp('Preparing the matrices')
amatinv=inv(amat); % we need it....
rstart=1-pts(:,1).^2; % starting data. 
% We use constant boundary conditions at 1, -1 also.
% plot(pts(:,1),rstart)
evalpts=(-1:0.005:1)'; % evaluation points
vmat=zeros(length(evalpts),n); % the v matrix = Lagrange functions
% The following commented stuff
% is just for some high-resolution checking of functions. 
% vdmat=zeros(length(evalpts),n); % the vx matrix = Lagrange functions once derived
% vddmat=zeros(length(evalpts),n); % the vxx matrix = Lagrange functions twice derived
for i=1:n
    for j=1:length(evalpts)
        z=kernel1D(evalpts(j,1),pts(i,1));
        vmat(j,i)=z(1);
        %         vdmat(j,i)=z(2);
        %         vddmat(j,i)=z(3);
    end
end
vmat=vmat*amatinv;
% vdmat=vdmat*amatinv;
% vddmat=vddmat*amatinv;
% % % now we plot a single case at about x=1/2
% % nex=floor(n/4);
% % figure
% % plot(evalpts,vmat(:,nex),...
% %     evalpts, vdmat(:,nex)/max(abs(vdmat(:,nex))),...
% %     evalpts,vddmat(:,nex)/max(abs(vddmat(:,nex))),...
% %     pts,zeros(n),'o')
% % title('Lagrange basis function and derivatives')
% % legend('Function','derivative','Second derivative','Points')
% % figure
% % v1=vmat(3:end,nex)-vmat(1:end-2,nex);
% % v1=v1/max(abs(v1));
% % v2=vdmat(2:end-1,nex)/max(abs(vdmat(:,nex)));
% % plot(evalpts(2:end-1),v1,evalpts(2:end-1),v2,pts,zeros(n),'o')
% % figure
% % v1=vdmat(3:end,nex)-vdmat(1:end-2,nex);
% % v1=v1/max(abs(v1));
% % v2=vddmat(2:end-1,nex)/max(abs(vddmat(:,nex)));
% % plot(evalpts(2:end-1),v1,evalpts(2:end-1),v2,pts,zeros(n),'o')
% 
% % this ends the diagnostic stuff
% We now have to calculate the much 
% smaller matrices on the interpolation points.
% Note that X in the paper just uses the interior point,
% while \Lambda stands for the two boundary functionals.
% We use X and L here.
vlocmat=zeros(n,n); % the v matrix = Lagrange functions
vdlocmat=zeros(n,n); % the vx matrix = Lagrange functions once derived
vddlocmat=zeros(n,n); % the vxx matrix = Lagrange functions twice derived
for i=1:n
    for j=1:n
        z=kernel1D(pts(i,1),pts(j,1));
        vlocmat(i,j)=z(1);
        vdlocmat(i,j)=z(2);
        vddlocmat(i,j)=z(3);
    end
end
vdlocmat=vdlocmat*amatinv;
vlocmat=vlocmat*amatinv;
vddlocmat=vddlocmat*amatinv;
% 
% figure
% nex=floor(n/4);
% plot(pts,vlocmat(:,nex),pts,zeros(n),'o')
% hold on;
% plot(pts,vdlocmat(:,nex)/max(abs(vdlocmat(:,nex))),'r')
% hold on;
% plot(pts,vddlocmat(:,nex)/max(abs(vddlocmat(nex,:))),'g')
% title('Lagrange basis function and derivatives')
boldg=[rstart(1,1);rstart(end,1)]; % like boldface g in the paper
VL=[vlocmat(:,1)'; vlocmat(:,end)']; % these are the matrices arising in the paper
VLd=[vdlocmat(:,1)'; vdlocmat(:,end)'];
VLdd=[vddlocmat(:,1)'; vddlocmat(:,end)'];
VX=vlocmat(:,2:(end-1))';
VXd=vdlocmat(:,2:(end-1))';
VXdd=vddlocmat(:,2:(end-1))';
% Note that we actually do not need the VL matrices 
% because the boundary values are zero.
% But for completeness wee added them in.

% This is the vector of starting variables
wstart=rstart(2:end-1,1);

% sval=VX'*wstart+VL'*boldg;
% sdval=VXd'*wstart+VLd'*boldg;
% sddval=VXdd'*wstart+VLdd'*boldg;
% figure
% plot(pts(:,1),sval)
% title('Interpolant of start data')
% figure
% plot(pts(:,1),sdval(:))
% title('Derivative of interpolant of start data')
% figure
% plot(pts(:,1),sddval(:))
% title('second derivative of interpolant of start data')
% rhs=sddval/Re-sval.*sdval;
% figure
% plot(pts(:,1),rhs(:))
% title('rhs')
disp('Starting the ODE solver')
% We now invoke an ODE solver,
% a stiff one, preferably 
% [t,y] = ode15s(@burgersfun,[0 0.6],wstart);
[t,y] = ode23s(@burgersfun,0:0.1:1.2,wstart);
% plot will be done on finer mesh, using interpolant
disp('Plotting')
figure
z=vmat*[zeros(length(t),1) y zeros(length(t),1)]';
for i=1:length(t)
    plot(evalpts(:,1),z(:,i))
    hold on
end 
